/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import Color from 'color';
import React, { ReactElement, useState } from 'react';
import { EntitiesFromSchema } from '../Types';
import styles from './PlotTitleStyles.module.css';
import { XYPosition } from 'reactflow';
import { getWidthOfText } from '@graphpolaris/shared/lib/schema/schema-utils';
import OptimizedAutocomplete from './OptimizedAutocomplete';

/** Props of the plots title component. */
type PlotTitleProps = {
  entity: string;
  attributeName: string;
  attributeValue: string;
  entitiesFromSchema: EntitiesFromSchema;
  nodeColor: string;
  pos: XYPosition;
  possibleAttrValues: string[];
  onTitleChanged(entity: string, attrName: string, attrValue: string): void;
};

/** State of the plots title component. */
type PlotTitleState = {
  isEditingAttrValue: boolean;

  // Dropdown when editing entity or attributeName
  menuAnchor: Element | null;
  menuItems: string[];
  menuItemsOnClick(value: string): void;
};
/**
 * A semantic substrates React component for rendering a plot title.
 * With functionality to change the entity, attributeType and attributeName.
 */
export default function PlotTitleComponent(props: PlotTitleProps) {
  const [state, setState] = useState<PlotTitleState>({
    isEditingAttrValue: false,
    menuAnchor: null,
    menuItems: [],
    menuItemsOnClick: () => true,
  });

  /**
   * Will be called when the value of the entity changes.
   * @param {string} value The new entity value.
   */
  function onEntityChanged(value: string): void {
    // Get the first attribute name for this entity.
    const attrName =
      props.entitiesFromSchema.attributesPerEntity[value].textAttributeNames.length > 0
        ? props.entitiesFromSchema.attributesPerEntity[value].textAttributeNames[0]
        : '?';

    props.onTitleChanged(value, attrName, '?');
  }

  /**
   * Will be called when the attribute name changes.
   * @param {string} value The new attribute name value.
   */
  function onAttrNameChanged(value: string): void {
    props.onTitleChanged(props.entity, value, '?');
  }

  /**
   * Will be called when the attribute value changes.
   * @param {string} value The new value of the attribute value.
   */
  function onAttrValueChanged(value: string): void {
    if (value == '') value = '?';

    // only update the state if the attribute value didn't change
    // If the attribute value did change, this component will be rerendered anyway
    if (value != props.attributeValue) props.onTitleChanged(props.entity, props.attributeName, value);
    else
      setState({
        ...state,
        isEditingAttrValue: false,
      });
  }

  /**
   * Renders the attribute value, either plain svg <text> element or, <input> if the user is editing this field.
   * @param {number} xOffset The x position offset.
   * @returns {ReactElement} The svg elements to render for the attribute value.
   */
  function renderAttributeValue(xOffset: number): ReactElement {
    if (state.isEditingAttrValue)
      return (
        <foreignObject x={xOffset} y="-16" width="200" height="150">
          <div>
            <OptimizedAutocomplete
              options={props.possibleAttrValues}
              currentValue={props.attributeValue}
              onLeave={(v: string) => onAttrValueChanged(v)}
            />
          </div>
        </foreignObject>
      );
    else
      return (
        <text
          x={xOffset}
          className={styles.clickable}
          onClick={() =>
            setState({
              ...state,
              isEditingAttrValue: true,
            })
          }
        >
          {props.attributeValue}
        </text>
      );
  }

  const { entity, attributeName, entitiesFromSchema, pos, nodeColor } = props;

  const withOffset = getWidthOfText(entity + ' ', 'arial', '15px', 'bold');
  const attrNameOffset = getWidthOfText('with ', 'arial', '15px') + withOffset;
  const colonOffset = getWidthOfText(attributeName + ' ', 'arial', '15px', 'bold') + attrNameOffset;
  const attrValueOffset = getWidthOfText(': ', 'arial', '15px', 'bold') + colonOffset;

  const nodeColorDarkened = Color(nodeColor).darken(0.3).hex();

  return (
    <g transform={`translate(${pos.x},${pos.y})`}>
      <text
        className={styles.clickable}
        fill={nodeColorDarkened}
        onClick={(event: React.MouseEvent<SVGTextElement>) => {
          setState({
            ...state,
            menuAnchor: event.currentTarget,
            menuItems: entitiesFromSchema.entityNames,
            menuItemsOnClick: (v: string) => onEntityChanged(v),
          });
        }}
      >
        {entity}
      </text>
      <text x={withOffset} fontSize={15}>
        with
      </text>
      <text
        className={styles.clickable}
        x={attrNameOffset}
        onClick={(event: React.MouseEvent<SVGTextElement>) => {
          setState({
            ...state,
            menuAnchor: event.currentTarget,
            menuItems: entitiesFromSchema.attributesPerEntity[entity].textAttributeNames,
            menuItemsOnClick: (v: string) => onAttrNameChanged(v),
          });
        }}
      >
        {attributeName}
      </text>
      <text x={colonOffset} fontWeight="bold" fontSize={15}>
        :
      </text>

      {renderAttributeValue(attrValueOffset)}

      <div id="simple-menu">
        {state.menuItems.map((option) => (
          <label
            key={option}
            onClick={() => {
              setState({
                ...state,
                menuAnchor: null,
                menuItems: [],
              });

              state.menuItemsOnClick(option);
            }}
          >
            {option}
          </label>
        ))}
      </div>
    </g>
  );
}
